@@ -86,7 +86,7 @@ from numpy.compat import asbytes, asstr
 
 import scipy.sparse
 
-from scipy.lib.six import string_types
+from scipy._lib.six import string_types
 
 from .byteordercodes import native_code, swapped_code
 
@@ -99,8 +99,8 @@ from .mio5_utils import VarReader5
 
 # Constants and helper objects
 from .mio5_params import (MatlabObject, MatlabFunction, MDTYPES, NP_TO_MTYPES,
-                          NP_TO_MXTYPES, miCOMPRESSED, miMATRIX, miINT8, miUTF8,
-                          miUINT32, mxCELL_CLASS, mxSTRUCT_CLASS,
+                          NP_TO_MXTYPES, miCOMPRESSED, miMATRIX, miINT8,
+                          miUTF8, miUINT32, mxCELL_CLASS, mxSTRUCT_CLASS,
                           mxOBJECT_CLASS, mxCHAR_CLASS, mxSPARSE_CLASS,
                           mxDOUBLE_CLASS, mclass_info)
 
@@ -400,6 +400,10 @@ def varmats_from_mat(file_obj):
     return named_mats
 
 
+class EmptyStructMarker(object):
+    """ Class to indicate presence of empty matlab struct on output """
+
+
 def to_writeable(source):
     ''' Convert input object ``source`` to something we can write
 
@@ -409,54 +413,11 @@ def to_writeable(source):
 
     Returns
     -------
-    arr : ndarray
-
-    Examples
-    --------
-    >>> to_writeable(np.array([1])) # pass through ndarrays
-    array([1])
-    >>> expected = np.array([(1, 2)], dtype=[('a', '|O8'), ('b', '|O8')])
-    >>> np.all(to_writeable({'a':1,'b':2}) == expected)
-    True
-    >>> np.all(to_writeable({'a':1,'b':2, '_c':3}) == expected)
-    True
-    >>> np.all(to_writeable({'a':1,'b':2, 100:3}) == expected)
-    True
-    >>> np.all(to_writeable({'a':1,'b':2, '99':3}) == expected)
-    True
-    >>> class klass(object): pass
-    >>> c = klass
-    >>> c.a = 1
-    >>> c.b = 2
-    >>> np.all(to_writeable({'a':1,'b':2}) == expected)
-    True
-    >>> to_writeable([])
-    array([], dtype=float64)
-    >>> to_writeable(())
-    array([], dtype=float64)
-    >>> to_writeable(None)
-
-    >>> to_writeable('a string').dtype.type == np.str_
-    True
-    >>> to_writeable(1)
-    array(1)
-    >>> to_writeable([1])
-    array([1])
-    >>> to_writeable([1])
-    array([1])
-    >>> to_writeable(object()) # not convertable
-
-    dict keys with legal characters are convertible
-
-    >>> to_writeable({'a':1})['a']
-    array([1], dtype=object)
-
-    but not with illegal characters
-
-    >>> to_writeable({'1':1}) is None
-    True
-    >>> to_writeable({'_a':1}) is None
-    True
+    arr : None or ndarray or EmptyStructMarker
+        If `source` cannot be converted to something we can write to a matfile,
+        return None.  If `source` is equivalent to an empty dictionary, return
+        ``EmptyStructMarker``.  Otherwise return `source` converted to an
+        ndarray with contents for writing to matfile.
     '''
     if isinstance(source, np.ndarray):
         return source
@@ -476,12 +437,12 @@ def to_writeable(source):
         for field, value in source.items():
             if (isinstance(field, string_types) and
                     field[0] not in '_0123456789'):
-                dtype.append((field,object))
+                dtype.append((field, object))
                 values.append(value)
         if dtype:
-            return np.array([tuple(values)],dtype)
+            return np.array([tuple(values)], dtype)
         else:
-            return None
+            return EmptyStructMarker
     # Next try and convert to an array
     narr = np.asanyarray(source)
     if narr.dtype.type in (np.object, np.object_) and \
@@ -610,7 +571,7 @@ class VarWriter5(object):
 
         Parameters
         ----------
-        arr : array-like
+        arr : array_like
             array-like object to create writer for
         name : str, optional
             name as it will appear in matlab workspace
@@ -630,7 +591,7 @@ class VarWriter5(object):
 
         Parameters
         ----------
-        arr : array-like
+        arr : array_like
             array-like object to create writer for
         '''
         # store position, so we can update the matrix tag
@@ -649,6 +610,8 @@ class VarWriter5(object):
             self.write_object(narr)
         elif isinstance(narr, MatlabFunction):
             raise MatWriteError('Cannot write matlab functions')
+        elif narr is EmptyStructMarker:  # empty struct array
+            self.write_empty_struct()
         elif narr.dtype.fields:  # struct array
             self.write_struct(narr)
         elif narr.dtype.hasobject:  # cell array
@@ -743,7 +706,8 @@ class VarWriter5(object):
                           mxSPARSE_CLASS,
                           is_complex=is_complex,
                           is_logical=is_logical,
-                          nzmax=nz)
+                          # matlab won't load file with 0 nzmax
+                          nzmax=1 if nz == 0 else nz)
         self.write_element(A.indices.astype('i4'))
         self.write_element(A.indptr.astype('i4'))
         self.write_element(A.data.real)
@@ -758,6 +722,13 @@ class VarWriter5(object):
         for el in A:
             self.write(el)
 
+    def write_empty_struct(self):
+        self.write_header((1, 1), mxSTRUCT_CLASS)
+        # max field name length set to 1 in an example matlab struct
+        self.write_element(np.array(1, dtype=np.int32))
+        # Field names element is empty
+        self.write_element(np.array([], dtype=np.int8))
+
     def write_struct(self, arr):
         self.write_header(matdims(arr, self.oned_as),
                           mxSTRUCT_CLASS)
@@ -844,7 +815,7 @@ class MatFile5Writer(object):
            ``name`` which will appear in the matlab workspace in file load, and
            ``contents`` is something writeable to a matlab file, such as a numpy
            array.
-        write_header : {None, True, False}
+        write_header : {None, True, False}, optional
            If True, then write the matlab file header before writing the
            variables.  If None (the default) then write the file header
            if we are at position 0 in the stream.  By setting False
