@@ -12,6 +12,10 @@
   #endif
 #endif
 
+#ifndef DL_IMPORT
+  #define DL_IMPORT(_T) _T
+#endif
+
 __PYX_EXTERN_C DL_IMPORT(int) wrap_PyUFunc_getfperr(void);
 
 #endif /* !__PYX_HAVE_API__scipy__special___ellip_harm_2 */
